#include <curses.h>
#include <ctype.h>
#include <strstream.h>
#include <stdlib.h>
#include <time.h>
#include <signal.h>

#include "spl.h"



extern bool cont;
extern vector < string > plist;

#define RELEASE "UNSTABLE"	// Can be UNSTABLE, DEBUG, OR PRODUCTION
#define STATUS_X 1		// the X co-ordinate to go to for a status line e.x "Skipping to song blah"
#define STATUS_Y 20		// the Y co-ordinate to go to for a status line. see above example

#ifdef DEBUG
extern ofstream dlog;
extern static int useful;	// I use it somewhere
#endif // DEBUG

void spl_hup_handle (int nothing)// Signal handler -- ignores SIGHUP
{
  return;
}


void
interface (const char filename[], bool list = false, bool ran = false,
	   int t_out = 2500, int delay = 2)
{
  /* This stuff is stuff we will need no matter
   * what happens..if we play one file, we need it.
   * If we play a playlist, we also need it
   */

  pid_t pid_of_splay;
  int *bob;			// DO NOT REMOVE THIS. IT IS USED IN THE call to waitpid()
  int opt;
  WINDOW *std_screen;
  chtype std_back = ' ';

  time_t t;
  srand (time (&t));		// Seed the random number generator

  /*  ncurses init stuff  */
  std_screen = initscr ();
  cbreak ();
  noecho ();
  nonl ();
  intrflush (stdscr, FALSE);
  keypad (stdscr, TRUE);
  // std_screen = initscr() is redundant, since stdscr is already defined..I didn't know that before

  /* My init stuff */
  timeout (t_out);		// We want getch() to timeout after t_out ms

  /* Ok, here we see if we are going to be using a playlist... */
  if (list == false)
    play (filename, pid_of_splay);
  else
    {
      /* 
         ifstream plist_file; // The playlist file
         plist_file.open(filename,ios::nocreate);
         if(plist_file.fail()){
         endwin();
         cout << endl << "Playlist does not exist! \n";
         return;
         }
         else{
         initlist(plist,filename, ran);
         }
       */
      initlist (plist, filename, ran);
    }

  if (list == false)
    {
      /* Here is the code for playing a single 
       * file. As you can tell, spl(ister) is 
       * really meant to do PLAYLISTS, not play
       * individual files -- there is no reason to
       * not use just splay for only playlist one file, since
       * spl(ister) focuses on playlist moving functions.
       * Therefore, single file mode sucks a bit :)
       */
      splashScreen ();
      showFileName (filename);
      while (!waitpid (pid_of_splay, bob, WNOHANG))
	sleep ((unsigned) 3);
    }
  else
    {
      ostrstream tstr;		// This is here because the destructor doesn't like me
      tstr << plist.size ();	// Put the size of the list in tstr

      splashScreen ();		// Intro message

      show_opt_bar ();		// Show the options bar

      timeout (t_out);		// Set the timeout

      for (int i = 0; i < plist.size () /* && plist[i].length() != 0 */ ; i++)
	{
	  signal(SIGHUP,spl_hup_handle);
	  wait(bob);
	  ostrstream ploc;
	  ploc << i + 1 << ends;
	  int err = 0;
	  err = play (plist[i].c_str (), pid_of_splay);
	  if (err == -1)
	    {
	      endwin ();
	      cout <<
		"Agh. Could not load a new copy of the audio function! Check free resources!\n";
	      return;
	    }


	  box (std_screen, 0, 0);	// Put a box around the screen
	  showFileName (plist[i].c_str ());	// Show the filename
	  /* Show where we are in the playlist */
	  move (19, 1);
	  clrtoeol ();
	  move (19, 1);
	  addstr ("Playing File : ");
	  addstr (ploc.str ());
	  refresh ();		// Refresh the display

	  ploc.freeze (0);	// We need to unfreeze ploc so it gets de-allocated correctly

#ifdef DEBUG			// if we are in debug mode
	  dlog << endl;
	  dlog << "DEBUG ENTRY FOR MAIN PLAYING LOOP:\n";
	  dlog << "The Variable i in the main program loop is equal to:\t"
	    << i << endl;
	  dlog << "The \"Playing File\" inidicator is set to :\t\t"
	    << ploc.str () << endl;
	  dlog << "The File playing is : " << plist[i].c_str ();
	  dlog << endl << endl;
	  ploc.freeze (0);	// Unfreeze ploc
	  useful = 0;		// Set useful counter to 0
#endif // DEBUG


	  show_opt_bar ();
	  box (std_screen, 0, 0);
	  char option = ' ';

	  while (!waitpid (pid_of_splay, bob, WNOHANG))
	    {
	      //wait(bob);
#ifdef DEBUG			// Debug code!
	      useful++;
	      dlog << endl;
	      dlog << "\tDEBUG ENTRY FOR WAITPID" << endl;
	      dlog << "\tThis is the __" << useful <<
		"__ time WAITPID has been called";
	      dlog << endl << endl;
#endif // Debug code

	      option = ' ';
	      option = getch ();
	      option = toupper (option);
	      cont = false;
	      switch (option)
		{
		case 'N':
		  move (STATUS_Y, STATUS_X);
		  addstr ("Skipping to the next song");
		  refresh ();

		  if (i == plist.size () - 1)
		    i = -1;
		  cont = true;	// We have to set cont to true no matter what!
		  break;
		case 'B':
		  move (STATUS_Y, STATUS_X);
		  addstr ("Skipping to the previous file");
		  refresh ();

		  if (i == 0)	// If we are at the beggining...
		    i = plist.size () - 2;
		  else
		    i = i - 2;
		  cont = true;
		  break;

		case 'S':
		  kill (pid_of_splay, SIGHUP);
		  waitpid (0, bob, WNOHANG);
		  move (STATUS_Y, STATUS_X);
		  addstr
		    ("Stopped. Press \'P\' to play again, or \'Q\' to Quit\n");
		  refresh ();
		  sleep ((unsigned) 1);
		  while ((option != 'P') && (option != 'Q'))
		    {
		      option = getch ();
		      option = toupper (option);
		    }
		  if (option == 'P')
		    {
		      play (plist[i].c_str (), pid_of_splay);
		      move (STATUS_Y, STATUS_X);
		      clrtoeol ();
		      box (std_screen, 0, 0);
		    }
		  else if (option == 'Q')
		    {
		      endwin ();
		      kill (0, SIGHUP);
		      sleep ((unsigned) 1);
		      return;
		    }
		  break;

		case 'Q':
		  clear ();
		  refresh ();
		  endwin ();
		  kill (pid_of_splay, SIGHUP);
		  sleep (delay);
		  return;
		  break;

		case 'R':
		  int c;

		  move (STATUS_Y, STATUS_X);
		  addstr ("Going to a random song");
		  refresh ();

		  i = rand () % plist.size () - 1;
		  cont = true;
		  break;

		case '1':
		  move (STATUS_Y, STATUS_X);
		  addstr ("Skipping 10 songs");
		  refresh ();

		  if (i >= plist.size () - 11)
		    i = -1;
		  else if (i == plist.size () - 2)	// If we are at the end
		    i = 9;	// We want track 10
		  else
		    i = i + 9;

		  cont = true;	// We have to set cont to true no matter what!
		  break;

		case '0':
		  move (STATUS_Y, STATUS_X);
		  addstr ("Skipping back 10 songs");
		  refresh ();

		  if (i == 0)
		    {		// If we are at the beggining...
		      /* for (c = plist.size () - 1; plist[c].length () == 0;
			 c--); */
		      i = plist.size () - 2;
		    }
		  else
		    i = i - 11;
		  cont = true;
		  break;

		case 'E':
		  move (STATUS_Y, STATUS_X);
		  addstr ("Going to the end of the play list");
		  refresh ();
		  i = plist.size () - 2;

		  cont = true;
		  break;
		case 'G':
		  move (STATUS_Y, STATUS_X);
		  addstr ("Going back to the begging of the playlist");
		  refresh ();

		  i = -1;
		  cont = true;
		  break;
		case 'O':
		  move (STATUS_Y, STATUS_X);
		  addstr ("Please enter the file to go to('c' to Cancel): ");
		  addstr ("(at most ");
		  addstr (tstr.str ());
		  tstr.freeze (0);
		  addstr (") ");
		  refresh ();
		  char getme[10];
		  getme[0] = ' ';
		  echo ();
		  timeout (-1);
		  getstr (getme);
		  timeout (t_out);
		  noecho ();
		  getme[0] = tolower (getme[0]);
		  if (getme[0] == 'c')
		    {
		      move (20, 1);
		      clrtoeol ();
		      break;
		    }
		  else if ((atoi (getme) - 1 > plist.size ())
			   || (atoi (getme) < 0))
		    {
		      move (STATUS_Y, STATUS_X);
		      clrtoeol ();
		      move (STATUS_Y, STATUS_X);
		      addstr
			("Number is out of range! Enter a normal number Next time");
		      refresh ();
		      getch ();
		      move (STATUS_Y, STATUS_X);
		      clrtoeol ();
		      refresh ();
		    }
		  else
		    {
		      i = atoi (getme) - 1;
		      i--;
		      cont = true;
		    }
		  break;

		case 'H':	/* This is just a demo code block! Real help will be added
				   * once I get the hang of creating and deleting windows
				 */
		  move (STATUS_Y, STATUS_X);
		  addstr ("Invoking Help");
		  WINDOW *hwin;
		  hwin = newwin (10, 20, 10, 10);
		  //box(hwin,0,0);
		  wrefresh (hwin);
		  sleep (2);
		  wmove (hwin, 1, 1);
		  waddstr (hwin,
			   "1010101010101010101010101010101010101010101010101010101010101010101010101010101010101");
		  int ii = 10;
		  for (int cc = 50; cc >= 0; cc--)
		    {
		      touchwin (std_screen);
		      refresh ();
		      mvwin (hwin, ii, cc);
		      wrefresh (hwin);
		      sleep (1);
		    }
		  delwin (hwin);
		  touchwin (std_screen);
		  move (20, 1);
		  clrtoeol ();
		  refresh ();
		  break;

		};

	      if (cont)
		{
		  kill (0, SIGHUP);
		  waitpid (0, bob, WNOHANG);	// Clean up zombie
		  sleep (delay);	// Wait for it to die

		  move (STATUS_Y, STATUS_X);
		  clrtoeol ();	// Clean the status message
		  refresh ();
		  cont = false;	// Make sure cont is not true next time

		  break;	// Get out of the while, so we can loop around again
		}		// End of the if(cont)
	    }			// End of the while..

	  kill (pid_of_splay, SIGHUP);	// Clean up time
	  waitpid (0, bob, WNOHANG);	// Get rid of the zombie
	  wait(bob);
	}			// End of the for loop
    }				// end of the else

  /* Here is where all the deallocation of any variables
   * that I created is performed. This is a just-in-case
   * thing. Sometimes an ostrstream should have been 
   * un-frozen earlier, but I might have forgetton and such,
   * so I just have it here just in case.
   */
  //tstr.freeze(0); // Unfreeze tstr if it is frozen
  clear ();
  endwin ();			// Return to normal console mode
}
